/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MDBFactory.java 1121 2006-09-27 08:51:06Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.container.mdb;

import org.objectweb.easybeans.api.EZBContainer;
import org.objectweb.easybeans.api.FactoryException;
import org.objectweb.easybeans.api.bean.EasyBeansMDB;
import org.objectweb.easybeans.api.bean.info.IBeanInfo;
import org.objectweb.easybeans.api.pool.PoolException;
import org.objectweb.easybeans.container.AbsFactory;
import org.objectweb.easybeans.container.info.MessageDrivenInfo;
import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.pool.JPool;
import org.objectweb.easybeans.pool.PoolEntryStatistics;
import org.objectweb.easybeans.pool.PoolFactory;
import org.objectweb.easybeans.rpc.api.EJBRequest;
import org.objectweb.easybeans.rpc.api.EJBResponse;

/**
 * This classes is reponsible to manage the MDB objects.<br />
 * Each MDB object (EasyBeansMDB) has a link to a MessageEndPoint object. The
 * internal message endpoint object is used by the resource adapter with the
 * help of the message end point factory.
 * @author Florent Benoit
 */
public class MDBFactory extends AbsFactory<EasyBeansMDB> implements PoolFactory<EasyBeansMDB, Long> {

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(MDBFactory.class);

    /**
     * Runtime information about the session bean.
     */
    private MessageDrivenInfo messageDrivenInfo = null;

    /**
     * Builds a new MDB factory with a given name and its container.
     * @param className name of this factory (name of class that is managed)
     * @param container the root component of this factory.
     * @throws FactoryException if class can't be loaded.
     */
    public MDBFactory(final String className, final EZBContainer container) throws FactoryException {
        super(className, container);
        setPool(new JPool<EasyBeansMDB, Long>(this));
    }

    /**
     * A request comes to the bean factory and needs to be handled.<br>
     * A response is done which contains the answer.
     * @param request the EJB request.
     * @return a response that have been processed by the factory.
     */
    @Override
    public EJBResponse rpcInvoke(final EJBRequest request) {
        // Not used by MDB : Invocation is done by Resource Adapter on Message
        // End Point
        return null;
    }

    /**
     * Do a local call on a method of this factory.
     * @param hash the hash of the method to execute.
     * @param methodArgs the arguments of the method
     * @param beanId the id of the bean that we want (stateful).
     * @return response container new id (if any) and value.
     */
    public EJBResponse localCall(final long hash, final Object[] methodArgs, final Long beanId) {
        // Not used by MDB : Invocation is done by Resource Adapter on Message
        // End Point
        return null;
    }

    /**
     * Stops the factory.
     */
    public void stop() {
        // remove pool
        try {
            getPool().stop();
        } catch (PoolException e) {
            logger.error("Problem when stopping the factory", e);
        }
    }

    /**
     * @return information of the current bean.
     */
    public IBeanInfo getBeanInfo() {
        return messageDrivenInfo;
    }

    /**
     * @return information of the current bean.
     */
    public MessageDrivenInfo getMessageDrivenInfo() {
        return messageDrivenInfo;
    }

    /**
     * Sets the information object for a session bean.
     * @param messageDrivenInfo information on the bean.
     */
    public void setMessageDrivenInfo(final MessageDrivenInfo messageDrivenInfo) {
        this.messageDrivenInfo = messageDrivenInfo;
    }

    /**
     * Creates an instance with the given hint.
     * @param clue a clue given by the Pool. Could be null.
     * @throws PoolException if instance cannot be created.
     * @return the created instance.
     */
    public EasyBeansMDB create(final Long clue) throws PoolException {
        EasyBeansMDB instance = null;
        try {
            instance = getBeanClass().newInstance();
        } catch (InstantiationException e) {
            throw new PoolException("Cannot create a new instance", e);
        } catch (IllegalAccessException e) {
            throw new PoolException("Cannot create a new instance", e);
        }

        // Set the factory
        instance.setEasyBeansFactory(this);

        // Init the MDB context
        EasyBeansMDBContext mdbContext = new EasyBeansMDBContext(instance);
        instance.setEasyBeansContext(mdbContext);

        ClassLoader oldClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(getContainer().getClassLoader());
        try {
            // Call injection
            injectResources(instance);

            // post construct callback
            instance.postConstructEasyBeansLifeCycle();
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassLoader);
        }

        return instance;
    }

    /**
     * Checks if the given object with the given clue is matching.
     * @param object given object against which the check should be done.
     * @param clue the object used as clue to check the matching.
     * @return true if it is matching, else false.
     */
    public boolean isMatching(final EasyBeansMDB object, final Long clue) {
        // all instances are matching.
        // But we shouldn't go here as we can use any instance.
        return true;
    }

    /**
     * Validate an instance by giving some statistics.
     * @param object the instance to validate
     * @param stats some statistics to help in the validating process.
     * @return true if the element is valid, else false.
     */
    public boolean validate(final EasyBeansMDB object, final PoolEntryStatistics stats) {
        return true;
    }


}
