/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: SessionFactory.java 1133 2006-10-04 14:30:41Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.container.session;

import org.objectweb.easybeans.api.EZBContainer;
import org.objectweb.easybeans.api.FactoryException;
import org.objectweb.easybeans.api.bean.EasyBeansSB;
import org.objectweb.easybeans.api.bean.info.IBeanInfo;
import org.objectweb.easybeans.api.container.EZBSessionContext;
import org.objectweb.easybeans.api.pool.PoolException;
import org.objectweb.easybeans.container.AbsFactory;
import org.objectweb.easybeans.container.info.SessionBeanInfo;
import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.rpc.api.EJBRequest;
import org.objectweb.easybeans.rpc.api.EJBResponse;

/**
 * This class manages the session bean and its creation/lifecycle.
 * @param <PoolType> the type of bean instance.
 * @author Florent Benoit
 */
public abstract class SessionFactory<PoolType extends EasyBeansSB<PoolType>> extends AbsFactory<PoolType> {

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(SessionFactory.class);

    /**
     * Session Desc (deployment info).
     */
    private SessionBeanInfo sessionBeanInfo = null;

    /**
     * Builds a new factory with a given name and its container.
     * @param className name of this factory (name of class that is managed)
     * @param container the root component of this factory.
     * @throws FactoryException if class can't be loaded.
     */
    public SessionFactory(final String className, final EZBContainer container) throws FactoryException {
        super(className, container);
    }

    /**
     * Stops the factory.
     */
    public void stop() {
        // remove pool
        try {
            getPool().stop();
        } catch (PoolException e) {
            logger.error("Problem when stopping the factory", e);
        }

    }

    /**
     * @return information of the current bean.
     */
    public IBeanInfo getBeanInfo() {
        return sessionBeanInfo;
    }

    /**
     * @return information of the current bean.
     */
    public SessionBeanInfo getSessionBeanInfo() {
        return sessionBeanInfo;
    }

    /**
     * Sets the information object for a session bean.
     * @param sessionBeanInfo information on the bean.
     */
    public void setSessionBeanInfo(final SessionBeanInfo sessionBeanInfo) {
        this.sessionBeanInfo = sessionBeanInfo;
    }

    /**
     * Gets a new ID or a null value.
     * @param beanId given id.
     * @return new id
     */
    protected abstract Long getId(final Long beanId);

    /**
     * Creates an instance with the given hint.
     * @param clue a clue given by the Pool. Could be null.
     * @throws PoolException if instance cannot be created.
     * @return the created instance.
     */
    public PoolType create(final Long clue) throws PoolException {
        PoolType instance = null;
        ClassLoader oldClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(getContainer().getClassLoader());
        try {
            try {
                instance = getBeanClass().newInstance();
            } catch (InstantiationException e) {
                throw new PoolException("Cannot create a new instance", e);
            } catch (IllegalAccessException e) {
                throw new PoolException("Cannot create a new instance", e);
            } catch (Exception e) {
                throw new PoolException("Cannot create a new instance", e);
            }
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassLoader);
        }

        // Set the factory
        instance.setEasyBeansFactory(this);

        // Init the session Context
        EZBSessionContext<PoolType> sessionContext = new EasyBeansSessionContext<PoolType>(instance);
        instance.setEasyBeansContext(sessionContext);

        oldClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(getContainer().getClassLoader());
        try {
            // Call injection
            injectResources(instance);

            // post construct callback
            instance.postConstructEasyBeansLifeCycle();
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassLoader);
        }

        return instance;
    }

    /**
     * A request comes to the bean factory and needs to be handled.<br>
     * A response is done which contains the answer.
     * @param request the EJB request.
     * @return a response that have been processed by the factory.
     */
    @Override
    public EJBResponse rpcInvoke(final EJBRequest request) {
        // get hash of the request
        long hash = request.getMethodHash();

        // Get Args (use context classloader for the Serialization)
        Object[] args = null;
        ClassLoader oldClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(getContainer().getClassLoader());
        try {
            args = request.getMethodArgs();
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassLoader);
        }

        // call the method
        return localCall(hash, args, request.getBeanId());

    }

    /**
     * Gets a bean for the given id.
     * @param beanId id of the expected bean.
     * @return a Stateless bean.
     * @throws IllegalArgumentException if bean is not found.
     */
    protected abstract PoolType getBean(final Long beanId) throws IllegalArgumentException;


    /**
     * Do a local call on a method of this factory.
     * @param hash the hash of the method to execute.
     * @param methodArgs the arguments of the method
     * @param beanId the id of the bean that we want (stateful).
     * @return response container new id (if any) and value.
     */
    public abstract EJBResponse localCall(final long hash, final Object[] methodArgs, final Long beanId);

}
