/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: InterceptorsClassResolver.java 848 2006-07-12 11:23:26Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployment.annotations.helper.bean;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.ejb.Remove;

import org.objectweb.asm.Type;
import org.objectweb.easybeans.container.session.stateful.interceptors.RemoveAlwaysInterceptor;
import org.objectweb.easybeans.container.session.stateful.interceptors.RemoveOnlyWithoutExceptionInterceptor;
import org.objectweb.easybeans.deployment.annotations.InterceptorType;
import org.objectweb.easybeans.deployment.annotations.JClassInterceptor;
import org.objectweb.easybeans.deployment.annotations.JMethod;
import org.objectweb.easybeans.deployment.annotations.exceptions.ResolverException;
import org.objectweb.easybeans.deployment.annotations.impl.JInterceptors;
import org.objectweb.easybeans.deployment.annotations.metadata.ClassAnnotationMetadata;
import org.objectweb.easybeans.deployment.annotations.metadata.EjbJarAnnotationMetadata;
import org.objectweb.easybeans.deployment.annotations.metadata.MethodAnnotationMetadata;
import org.objectweb.easybeans.naming.interceptors.ENCManager;
import static org.objectweb.easybeans.deployment.annotations.helper.bean.InheritanceInterfacesHelper.JAVA_LANG_OBJECT;
/**
 * This class sets the EasyBeans interceptors used when invoking business methods and also for life cycle events.
 * @author Florent Benoit
 */
public final class InterceptorsClassResolver {

    /**
     * Signature of EasyBeans interceptors.
     */
    private static final JMethod EASYBEANS_INTERCEPTOR = new JMethod(0, "intercept",
            "(Lorg/objectweb/easybeans/api/EasyBeansInvocationContext;)Ljava/lang/Object;",
            null, new String[] {"java/lang/Exception"});


    /**
     * Helper class, no public constructor.
     */
    private InterceptorsClassResolver() {

    }


    /**
     * Found all interceptors of the class (including business and lifecycle events) and also set EasyBeans interceptors.
     * @param classAnnotationMetadata class to analyze
     * @throws ResolverException if metadata is missing
     */
    public static void resolve(final ClassAnnotationMetadata classAnnotationMetadata) throws ResolverException {

        // First, EasyBeans interceptors
        List<String> easyBeansInterceptorsClasses = new ArrayList<String>();
        // Add EasyBeans interceptors for all methods
        easyBeansInterceptorsClasses.add(Type.getInternalName(ENCManager.getInterceptorClass()));
        // Set list of global interceptors (applied on all business methods)
        List<JClassInterceptor> easyBeansGlobalInterceptors = new ArrayList<JClassInterceptor>();
        for (String easyBeansInterceptor : easyBeansInterceptorsClasses) {
            easyBeansGlobalInterceptors.add(new JClassInterceptor(easyBeansInterceptor, EASYBEANS_INTERCEPTOR));
        }
        classAnnotationMetadata.setGlobalEasyBeansInterceptors(easyBeansGlobalInterceptors);

        // TODO: adds the default interceptor (set in the DD)


        // And then, set the user interceptors (found in external class)
        List<String> externalInterceptorsClasses = new ArrayList<String>();

        // Interceptors from other classes (will be analyzed after)
        // See 3.5.3 of simplified EJB 3.0 spec : multiple interceptors
        // Add interceptor classes found on super classes (if any)


        // Invert list of the inheritance on the current class
        LinkedList<ClassAnnotationMetadata> invertedInheritanceClassesList =
            getInvertedSuperClassesMetadata(classAnnotationMetadata);
        // Add interceptors found on these classes (order is super super class
        // before super class, ie : top level first)
        for (ClassAnnotationMetadata superMetaData : invertedInheritanceClassesList) {
            JInterceptors classIinterceptors = superMetaData.getAnnotationInterceptors();
            if (classIinterceptors != null) {
                for (String cls : classIinterceptors.getClasses()) {
                    externalInterceptorsClasses.add(cls);
                }
            }
        }


        // Get the interceptors defined by the user on external classes
        Map<InterceptorType, List<JClassInterceptor>> externalMapClassInterceptors =
            getInterceptors(classAnnotationMetadata.getClassName(), classAnnotationMetadata.getEjbJarAnnotationMetadata(),
                    externalInterceptorsClasses);
        classAnnotationMetadata.setExternalUserInterceptors(externalMapClassInterceptors);


        // interceptor in the bean class ? (LifeCycle event interceptors are not in the bean
        // because they don't take an InvocationContext as parameter, this is the intercepted method
        List<String> internalInterceptorsClasses = new ArrayList<String>();

        if (classAnnotationMetadata.isAroundInvokeMethodMetadata()) {
            internalInterceptorsClasses.add(classAnnotationMetadata.getClassName());
        }
        // Get the interceptors defined by the user on the class
        Map<InterceptorType, List<JClassInterceptor>> internalMapClassInterceptors =
            getInterceptors(classAnnotationMetadata.getClassName(), classAnnotationMetadata.getEjbJarAnnotationMetadata(),
                    internalInterceptorsClasses);
        classAnnotationMetadata.setInternalUserInterceptors(internalMapClassInterceptors);


        // Now, analyze each interceptors found on methods.
        for (MethodAnnotationMetadata methodAnnotationMetaData
                : classAnnotationMetadata.getMethodAnnotationMetadataCollection()) {

            // Set global interceptors for a given method (ie : Remove)
            Remove remove = methodAnnotationMetaData.getJRemove();
            if (remove != null) {
                List<JClassInterceptor> easyBeansMethodGlobalInterceptors = new ArrayList<JClassInterceptor>();
                String classType = null;
                // choose right interceptor class
                if (remove.retainIfException()) {
                    classType = Type.getInternalName(RemoveOnlyWithoutExceptionInterceptor.class);
                } else {
                    classType = Type.getInternalName(RemoveAlwaysInterceptor.class);
                }
                easyBeansMethodGlobalInterceptors.add(new JClassInterceptor(classType, EASYBEANS_INTERCEPTOR));

                // set list
                methodAnnotationMetaData.setGlobalEasyBeansInterceptors(easyBeansMethodGlobalInterceptors);
            }

            JInterceptors methodAnnotationInterceptors = methodAnnotationMetaData.getAnnotationInterceptors();
            if (methodAnnotationInterceptors != null) {
                List<String> methodInterceptorsClasses = new ArrayList<String>();
                for (String cls : methodAnnotationInterceptors.getClasses()) {
                    methodInterceptorsClasses.add(cls);
                }
                Map<InterceptorType, List<JClassInterceptor>> mapMethodInterceptors =
                    getInterceptors(classAnnotationMetadata.getClassName() + "/Method "
                            + methodAnnotationMetaData.getMethodName(),
                            classAnnotationMetadata.getEjbJarAnnotationMetadata(), methodInterceptorsClasses);
                methodAnnotationMetaData.setUserInterceptors(mapMethodInterceptors);
            }

        }

    }


    /**
     * Found interceptors method in the given class. It will analyze each interceptor class and
     * fill a structure with a mapping between the annotation type and the corresponding interceptors.
     * @param referencingName name of the class/method that reference these interceptors
     * @param ejbJarAnnotationMetadata root of all annotations (used to find the metadata)
     * @param interceptorsClasses list of classes that contains interceptors
     * @return the map between the type of interceptor (PostConstrut, AroundInvoke, ...) and the JClassInterceptor objects
     * @throws ResolverException if analyze fails
     */
    private static Map<InterceptorType, List<JClassInterceptor>> getInterceptors(final String referencingName,
            final EjbJarAnnotationMetadata ejbJarAnnotationMetadata,
            final List<String> interceptorsClasses) throws ResolverException {
        // Define the mapping object.
        Map<InterceptorType, List<JClassInterceptor>> mapInterceptors = new HashMap<InterceptorType, List<JClassInterceptor>>();
        // Init the map for each interceptor type
        for (InterceptorType type : InterceptorType.values()) {
            mapInterceptors.put(type, new ArrayList<JClassInterceptor>());
        }

        int interceptorClassAnalyzed = 0;

        // For each interceptors classes, take the method with @AroundInvoke or @PostConstruct, etc. and build list
        for (String className : interceptorsClasses) {
            ClassAnnotationMetadata interceptorMetadata = ejbJarAnnotationMetadata.getClassAnnotationMetadata(className);
            if (interceptorMetadata == null) {
                throw new ResolverException("No medata for interceptor class " + className
                        + " referenced by " + referencingName);
            }

            // Another interceptor class
            interceptorClassAnalyzed++;

            // No inner class of the bean
            if (interceptorMetadata.getClassName().contains("$")) {
                throw new IllegalStateException("Interceptor can't be defined in an inner class.");
            }


            // Takes all methods of the super class and add them to the current class.
            InheritanceMethodResolver.resolve(interceptorMetadata);


            // Invert list of the inheritance on the current class
            LinkedList<ClassAnnotationMetadata> invertedInheritanceClassesList =
                getInvertedSuperClassesMetadata(interceptorMetadata);

            // For each class (starting super class first, add the interceptor methods)
            for (ClassAnnotationMetadata currentMetaData : invertedInheritanceClassesList) {
            // Analyze methods of the interceptor meta-data and add it in the map
                for (MethodAnnotationMetadata method : currentMetaData.getMethodAnnotationMetadataCollection()) {
                    // Don't look inherited methods.
                    if (method.isInherited()) {
                        continue;
                    }
                    JClassInterceptor jInterceptor = new JClassInterceptor(className, method.getJMethod(),
                            interceptorClassAnalyzed);

                    // If the method is overriden, take care of using the
                    // annotation of the lower class in the inheritance classes.
                    // As the method is only add once for a single interceptor
                    // class.
                    MethodAnnotationMetadata analyzedMethod = method;
                    MethodAnnotationMetadata methodSubClass = interceptorMetadata.getMethodAnnotationMetadata(method
                            .getJMethod());
                    if (methodSubClass != null) {
                        analyzedMethod = methodSubClass;
                    }


                    // A method can be designed to run for all annotation, so no "else if" !
                    if (analyzedMethod.isAroundInvoke()) {
                        addOnlyIfNotPresent(mapInterceptors.get(InterceptorType.AROUND_INVOKE), jInterceptor);
                    }
                    // Only if interceptor class is not a bean's class. Else, it is only simple methods.
                    if (!currentMetaData.isBean()) {
                        // build interceptor object.
                        if (analyzedMethod.isPostActivate()) {
                            addOnlyIfNotPresent(mapInterceptors.get(InterceptorType.POST_ACTIVATE), jInterceptor);
                        }
                        if (analyzedMethod.isPostConstruct()) {
                            addOnlyIfNotPresent(mapInterceptors.get(InterceptorType.POST_CONSTRUCT), jInterceptor);
                        }
                        if (analyzedMethod.isPreDestroy()) {
                            addOnlyIfNotPresent(mapInterceptors.get(InterceptorType.PRE_DESTROY), jInterceptor);
                        }
                        if (analyzedMethod.isPrePassivate()) {
                            addOnlyIfNotPresent(mapInterceptors.get(InterceptorType.PRE_PASSIVATE), jInterceptor);
                        }
                    }
                }
            }

        }




        return mapInterceptors;
    }

    /**
     * Adds in the given interceptors list the interceptor object.
     * If the object is already present in the list, doesn't add it again.
     * @param interceptors the list of interceptors.
     * @param jInterceptor the interceptor to add.
     */
    private static void addOnlyIfNotPresent(final List<JClassInterceptor> interceptors, final JClassInterceptor jInterceptor) {
        if (!interceptors.contains(jInterceptor)) {
            interceptors.add(jInterceptor);
        }
    }

    /**
     * Gets the inverted list of metadata for a given class (super class is the first one in the list).
     * @param classAnnotationMetadata the class to analyze
     * @return the given list
     */
    private static LinkedList<ClassAnnotationMetadata> getInvertedSuperClassesMetadata(
            final ClassAnnotationMetadata classAnnotationMetadata) {

        // get list of super classes
        LinkedList<ClassAnnotationMetadata> superClassesList = new LinkedList<ClassAnnotationMetadata>();
        String superClassName = classAnnotationMetadata.getSuperName();
        // loop while super class is not java.lang.Object
        while (!JAVA_LANG_OBJECT.equals(superClassName)) {
            ClassAnnotationMetadata superMetaData = classAnnotationMetadata.getEjbJarAnnotationMetadata()
                    .getClassAnnotationMetadata(superClassName);
            if (superMetaData != null) {
                superClassName = superMetaData.getSuperName();
                superClassesList.addFirst(superMetaData);
            } else {
                superClassName = JAVA_LANG_OBJECT;
            }
        }
        superClassesList.addLast(classAnnotationMetadata);
        return superClassesList;
    }

}
