/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EasyBeansClassLoader.java 534 2006-05-29 13:46:27Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.loader;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.HashMap;
import java.util.Map;

import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;

/**
 * This class defines the EasyBeans classloader. This classloader allows to set
 * the bytecode for a given class. Then, when the class will be loaded, it will
 * define the class by using the associated bytecode.
 * @author Florent Benoit
 */
public class EasyBeansClassLoader extends URLClassLoader implements Cloneable {

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(EasyBeansClassLoader.class);

    /**
     * Need to recompute toString() value ? (urls have changed)
     * True by default (not done).
     * Then, compute is done only when required and if needed
     */
    private boolean recomputeToString = true;

    /**
     * String representation used by toString() method.
     */
    private String toStringValue = null;

    /**
     * java.lang.ClassLoader class.
     */
    private Class javaLangClassLoaderClass = null;

    /**
     * Method of the classloader allowing to define some classes.
     */
    private Method defineClassMethod = null;

    /**
     * Map between class name and the bytecode associated to the given
     * classname.
     */
    private Map<String, byte[]> mapDefined = new HashMap<String, byte[]>();

    /**
     * Use the same constructors as parent class.
     * @param urls the URLs from which to load classes and resources
     * @param parent the parent class loader for delegation
     */
    public EasyBeansClassLoader(final URL[] urls, final ClassLoader parent) {
        super(urls, parent);
    }

    /**
     * Use the same constructors as parent class.
     * @param urls the URLs from which to load classes and resources
     */
    public EasyBeansClassLoader(final URL[] urls) {
        super(urls);
    }

    /**
     * Finds and loads the class with the specified name from the URL search
     * path. If this classloader has the bytecode for the associated class, it
     * defines the class.
     * @param name the name of the class
     * @return the resulting class
     * @exception ClassNotFoundException if the class could not be found
     */
    @Override
    protected Class<?> findClass(final String name) throws ClassNotFoundException {
        Class clazz = searchingDefinedClass(name);
        if (clazz != null) {
            return clazz;
        }
        // else, delegates to super class
        return super.findClass(name);
    }

    /**
     * Defines the class by using the bytecode of the given classname.
     * @param className the name of the class.
     * @param bytecode the bytes of the given class.
     * @return the class which is now defined
     */
    private Class<?> defineInternalClass(final String className, final byte[] bytecode) {
        // TODO: remove this ?
       // if (logger.isDebugEnabled()) {
            FileOutputStream fos = null;
            try {
                String fName = System.getProperty("java.io.tmpdir") + File.separator + className + ".class";
                fos = new FileOutputStream(fName);
                fos.write(bytecode);
            } catch (IOException ioe) {
                throw new RuntimeException(ioe);
            }
        //}

        // override classDefine (as it is protected) and define the class.
        ClassLoader loader = this;
        if (javaLangClassLoaderClass == null) {
            try {
                javaLangClassLoaderClass = Class.forName("java.lang.ClassLoader");
            } catch (ClassNotFoundException e) {
                logger.error("Cannot use the ClassLoader class", e);
                return null;
            }
        }
        if (defineClassMethod == null) {
            try {
                defineClassMethod = javaLangClassLoaderClass.getDeclaredMethod("defineClass", new Class[] {
                        String.class, byte[].class, int.class, int.class});
            } catch (SecurityException e) {
                logger.error("Method defineClass not available in the classloader class", e);
                return null;
            } catch (NoSuchMethodException e) {
                logger.error("Method defineClass not available in the classloader class", e);
                return null;
            }
        }
        // protected method invocaton
        defineClassMethod.setAccessible(true);
        try {
            Object[] args = new Object[] {className, bytecode, new Integer(0), new Integer(bytecode.length)};
            try {
                return (Class) defineClassMethod.invoke(loader, args);
            } catch (IllegalArgumentException e) {
                logger.error("Cannot invoke the defineClass method on the classloader", e);
            } catch (IllegalAccessException e) {
                logger.error("Cannot invoke the defineClass method on the classloader", e);
            } catch (InvocationTargetException e) {
                logger.error("Cannot invoke the defineClass method on the classloader", e);
            }
        } finally {
            defineClassMethod.setAccessible(false);
        }
        return null;
    }

    /**
     * Adds the bytecode for a given class. It will be used when class will be
     * loaded.
     * @param className the name of the class.
     * @param bytecode the bytes of the given class.
     */
    public void addClassDefinition(final String className, final byte[] bytecode) {
        // check override ?
        if (mapDefined.get(className) != null) {
            logger.warn("There is already a bytecode defined for the class named '" + className
                    + "'. Not replacing. This could be due to a duplicated class in the given package.");
        }
        mapDefined.put(className, bytecode);
    }

    /**
     * When trying to load a class, look if this class needs to be defined.
     * @param name the class name to load.
     * @return the class if it was found.
     * @throws ClassNotFoundException if the class is not found.
     */
    @Override
    public Class<?> loadClass(final String name) throws ClassNotFoundException {
        searchingDefinedClass(name);
        return super.loadClass(name, false);
    }

    /**
     * Search a class in the local repository of classes to define.
     * @param className the name of the class to search and define if found.
     * @return the class if it was defined and loaded.
     */
    private Class searchingDefinedClass(final String className) {
        // Defines the class if the bytecode is here.
        if (mapDefined != null) {
            byte[] defined = mapDefined.get(className);
            if (defined != null) {
                Class clazz = defineInternalClass(className, defined);
                if (clazz != null) {
                    // remove defined class.
                    mapDefined.remove(className);
                    return clazz;
                }

            }
        }
        return null;
    }


    /**
     * Displays useful information.
     * @return information
     */
    @Override
    public String toString() {
        // urls have changed, need to build value
        if (recomputeToString) {
            computeToString();
        }
        return toStringValue;
    }

    /**
     * Compute a string representation used by toString() method.
     */
    private void computeToString() {
        StringBuffer sb = new StringBuffer();
        sb.append(this.getClass().getName());
        sb.append("[");
        sb.append("urls=");
        URL[] urls = getURLs();
        for (int u = 0; u < urls.length; u++) {
            sb.append(urls[u]);
            if (u != urls.length - 1) {
                sb.append(";");
            }
        }
        sb.append("]");
        toStringValue = sb.toString();

        // value is updated, no need to do it again.
        recomputeToString = false;
   }

    /**
     * Creates and returns a copy of this object.
     * It is used for example when Persistence Provider needs a new Temp classloader to load some temporary classes.
     * @return a copy of this object
     */
    @Override
    public Object clone() {
        return new EasyBeansClassLoader(getURLs(), this.getParent());
    }
}
