/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Embedded.java 1164 2006-10-18 13:43:42Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.server;

import java.io.File;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.objectweb.easybeans.api.EZBArchive;
import org.objectweb.easybeans.api.EZBContainer;
import org.objectweb.easybeans.api.EZBContainerConfig;
import org.objectweb.easybeans.api.EZBServer;
import org.objectweb.easybeans.component.ComponentManager;
import org.objectweb.easybeans.component.Components;
import org.objectweb.easybeans.component.api.EZBComponent;
import org.objectweb.easybeans.component.api.EZBComponentException;
import org.objectweb.easybeans.component.itf.RegistryComponent;
import org.objectweb.easybeans.container.JContainer3;
import org.objectweb.easybeans.container.JContainerConfig;
import org.objectweb.easybeans.deployer.Deployer;
import org.objectweb.easybeans.jmx.CommonsModelerException;
import org.objectweb.easybeans.jmx.CommonsModelerHelper;
import org.objectweb.easybeans.jmx.JMXRemoteException;
import org.objectweb.easybeans.jmx.JMXRemoteHelper;
import org.objectweb.easybeans.jmx.MBeanServerException;
import org.objectweb.easybeans.jmx.MBeanServerHelper;
import org.objectweb.easybeans.jmx.MBeansException;
import org.objectweb.easybeans.jmx.MBeansHelper;
import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.rpc.rmi.server.RMIServerRPC;
import org.objectweb.easybeans.rpc.rmi.server.RMIServerRPCImpl;
import org.objectweb.easybeans.security.jacc.PolicyProvider;

/**
 * Allows to create an embedded EJB3 server.
 * @author Florent Benoit
 */
public class Embedded implements EZBServer {

    /**
     * Default sleep value (for server loop).
     */
    private static final int SLEEP_VALUE = 10000;

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(Embedded.class);

    /**
     * Internal (global) counter of all embedded instance created.
     */
    private static int counter = 0;

    /**
     * ID of this embedded server.
     */
    private Integer id = null;

    /**
     * Configuration of this server.
     */
    private ServerConfig serverConfig = null;

    /**
     * Map of managed ejb3 containers.
     */
    private Map<String, EZBContainer> containers = null;

    /**
     * List of Callbacks factories.
     */
    private List<EasyBeansConfigurationExtension> extensionFactories = null;

    /**
     * Deployer instance.
     */
    private Deployer deployer = null;

    /**
     * List of components that have been defined.
     */
    private Components components = null;

    /**
     * Manager of components.
     */
    private ComponentManager componentManager = null;

    /**
     * Server started ?
     */
    private boolean started = false;

    /**
     * Monitor of the containers.
     */
    private ContainersMonitor monitor = null;

    /**
     * Creates a new Embedded server.<br>
     * It will take default values of configuration.
     */
    public Embedded() {
        this.id = new Integer(counter++);
        this.containers = new HashMap<String, EZBContainer>();
        this.extensionFactories = new ArrayList<EasyBeansConfigurationExtension>();
        this.serverConfig = new ServerConfig();
        // register to the list
        EmbeddedManager.addEmbedded(this);
    }

    /**
     * Starts the EJB3 server.
     * @throws EmbeddedException if there is a failure while starting the
     *         server.
     */
    public void start() throws EmbeddedException {
        long tStart = System.currentTimeMillis();

        // Init JACC
        if (serverConfig.initJACC()) {
            PolicyProvider.init();
        }


        // configure
        configure();

        // Define component manager
        componentManager = new ComponentManager(components);

        // Init components
        try {
            componentManager.initComponents();
        } catch (EZBComponentException e) {
            throw new EmbeddedException("Cannot init components", e);
        }

        // Start components
        try {
            componentManager.startComponents();
        } catch (EZBComponentException e) {
            throw new EmbeddedException("Cannot start components", e);
        }

        // Use ExtensionFactories ?
        if (!serverConfig.getExtensionFactories().isEmpty()) {
            List<String> classnames = serverConfig.getExtensionFactories();
            for (String fqn : classnames) {
                EasyBeansConfigurationExtension factory = null;
                try {
                    factory = Class.forName(fqn).asSubclass(EasyBeansConfigurationExtension.class).newInstance();
                } catch (Exception e) {
                    throw new EmbeddedException("Cannot instantiate CallbackFacory : " + fqn, e);
                }
                this.extensionFactories.add(factory);
            }
        }

        if (serverConfig.isUsingNaming()) {
            // url.pkg for java:
            System.setProperty(Context.URL_PKG_PREFIXES, "org.objectweb.easybeans.naming.pkg");
        }

        // set the deployer used for this component.
        deployer = new Deployer(this);

        MBeansHelper.getInstance().activate(serverConfig.isUsingMBeans());
        if (serverConfig.isUsingMBeans()) {

            // init Modeler Registry
            try {
                CommonsModelerHelper.initRegistry();
            } catch (CommonsModelerException e) {
                throw new EmbeddedException("Cannot init MBean server", e);
            }

            // Start MBeanServer (if any)
            try {
                MBeanServerHelper.startMBeanServer();
            } catch (MBeanServerException e) {
                throw new EmbeddedException("Cannot start MBean server", e);
            }

            try {
                JMXRemoteHelper.startConnector((RegistryComponent) this
                        .getComponent("org.objectweb.easybeans.component.carol.CarolComponent"));
            } catch (JMXRemoteException e) {
                throw new EmbeddedException("Cannot start JMX Remote connector", e);
            }

            // register the Deployer
            try {
                MBeansHelper.getInstance().registerMBean(deployer);
            } catch (MBeansException e) {
                throw new EmbeddedException("Cannot init MBeans", e);
            }

            // register the Server
            try {
                MBeansHelper.getInstance().registerMBean(this);
            } catch (MBeansException e) {
                throw new EmbeddedException("Cannot init MBeans", e);
            }
        }

        // Bind the RPC object
        RMIServerRPC invoker = null;
        try {
            invoker = new RMIServerRPCImpl(this);
        } catch (RemoteException e) {
            throw new EmbeddedException("Cannot build RPC invoker", e);
        }
        try {
            new InitialContext().rebind(RMIServerRPC.RPC_JNDI_NAME, invoker);
        } catch (NamingException e) {
            throw new EmbeddedException("Cannot bind the RPC invoker", e);
        }


        if (serverConfig.isDirectoryScanningEnabled()) {
            // add a thread which monitor containers file
            monitor = new ContainersMonitor(this);
            monitor.init();
        }

        logger.info("Embedded.start.startup", Version.VERSION, Long.valueOf(System.currentTimeMillis() - tStart));
        logger.info("Embedded.start.created", Integer.valueOf(containers.size()));

        logger.info("Embedded.start.waiting");
        if (serverConfig.isDirectoryScanningEnabled()) {
            monitor.start();
        }

        // It is started
        started = true;

        if (serverConfig.shouldWait()) {
            while (true) {
                try {
                    Thread.sleep(SLEEP_VALUE);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * Stops the EJB3 server.
     * @throws EmbeddedException if container cannot be stopped.
     */
    public synchronized void stop() throws EmbeddedException {
        // ensure started
        if (!started) {
            throw new EmbeddedException("Cannot stop the server as it is not started.");
        }

        // Stop the containers
        // Use a ListIterator to allow us to safely remove EZBContainers
        // from the List being processed.
        List<EZBContainer> containersList = new ArrayList<EZBContainer>(this.containers.values());
        ListIterator<EZBContainer> li = containersList.listIterator();
        while (li.hasNext()) {
            EZBContainer container = li.next();
            container.stop();
            removeContainer(container);
        }
        monitor.stopOrder();

        // Unregister MBeans
        if (serverConfig.isUsingMBeans()) {

            // Stop the JMX Connector
            try {
                JMXRemoteHelper.stopConnector();
            } catch (JMXRemoteException e) {
                // Only log the Exception
                logger.error("Cannot stop JMX Remote connector", e);
                // and continue ...
            }

            // Unregister the Deployer
            try {
                MBeansHelper.getInstance().unregisterMBean(deployer);
            } catch (MBeansException e) {
                // Only log the Exception
                logger.error("Cannot unregister Deployer MBean", e);
            }

            // Unregister the Server
            try {
                MBeansHelper.getInstance().unregisterMBean(this);
            } catch (MBeansException e) {
                // Only log the Exception
                logger.error("Cannot unregister Embedded MBean", e);
            }
        }

        // Unbind the RPCInvoker Remote Object
        try {
            new InitialContext().unbind(RMIServerRPC.RPC_JNDI_NAME);
        } catch (NamingException e) {
            // Only log the Exception
            logger.error("Cannot unbind the RPC invoker", e);
        }

        // Stop the components
        componentManager.stopComponents();

        logger.info("Embedded.stop.stopped", Version.VERSION);
    }

    /**
     * Sets the server configuration (not the components).
     * @param serverConfig the given configuration
     */
    public void setServerConfig(final ServerConfig serverConfig) {
        // check status
        if (started) {
            throw new IllegalStateException("Cannot set the server configuration when server has been started.");
        }
        this.serverConfig = serverConfig;
    }

    /**
     * Gets a container managed by this server.
     * @param id the container id.
     * @return the container if it is found, else null.
     */
    public EZBContainer getContainer(final String id) {
        return containers.get(id);
    }

    /**
     * Configure the server by using the given configuration.
     */
    private void configure() {
        File ejb3Directory = serverConfig.getEjb3Directory();
        // need to define a repository
        if (ejb3Directory == null) {
            ejb3Directory = new File("ejb3s");
            logger.debug("No directory was configured, take the default value of '" + ejb3Directory.getAbsolutePath() + "'.");
            if (!ejb3Directory.exists()) {
                if (ejb3Directory.mkdir()) {
                    logger.warn("Directory '" + ejb3Directory.getAbsolutePath() + "' created.");
                } else {
                    throw new IllegalStateException("Cannot make directory '" + ejb3Directory.getAbsolutePath() + "'.");
                }
            }
            // update the value
            serverConfig.setEjb3Path(ejb3Directory.getAbsolutePath());
        }
    }

    /**
     * Creates and adds an ejb3 container to the managed container.
     * @param archive jar file or exploded archive.
     * @return the created container.
     */
    public EZBContainer createContainer(final EZBArchive archive) {
        EZBContainerConfig jConfig = new JContainerConfig(archive);
        jConfig.setEZBServer(this);
        callJContainerConfigExtensions(jConfig);
        EZBContainer container = new JContainer3(jConfig);

        addContainer(container);

        return container;
    }

    /**
     * Add an already created container.
     * @param container the EZBContainer to be added.
     */
    public void addContainer(final EZBContainer container) {
        String id = container.getId();
        containers.put(id, container);

        try {
            MBeansHelper.getInstance().registerMBean(container);
        } catch (MBeansException e) {
            // TODO what to do here ? log or exception ?
            logger.error("Cannot register Container MBeans for " + container.getArchive().getName(), e);
        }
    }

    /**
     * Remove a given container.
     * @param container the container to be removed.
     */
    public void removeContainer(final EZBContainer container) {
        containers.remove(container.getId());

        try {
            MBeansHelper.getInstance().unregisterMBean(container);
        } catch (MBeansException e) {
            // TODO what to do here ? log or exception ?
            logger.error("Cannot unregister Container MBeans for " + container.getArchive().getName(), e);
        }
    }

    /**
     * Adapt the JContainerConfig for all
     * {@link EasyBeansConfigurationExtension}.
     * @param jcc the JContainerConfig to adapt.
     */
    private void callJContainerConfigExtensions(final EZBContainerConfig jcc) {

        for (EasyBeansConfigurationExtension extension : this.extensionFactories) {
            try {
                extension.configure(jcc);
            } catch (Throwable t) {
                // prevent malicious code to break everything ...
                logger.info("Failed to configure JContainerConfig with {0}", extension.getClass().getName());
            }
        }
    }

    /**
     * @return the configuration of this server.
     */
    public ServerConfig getServerConfig() {
        return serverConfig;
    }

    /**
     * @return the containers managed by this server.
     */
    public Map<String, EZBContainer> getContainers() {
        return containers;
    }

    /**
     * Gets the id of this embedded server.
     * @return the id of this server.
     */
    public Integer getID() {
        return id;
    }

    /**
     * @return Returns the linked Deployer instance.
     */
    public Deployer getDeployer() {
        return deployer;
    }

    /**
     * Gets the components that have been defined for this embedded server.
     * @return the components.
     */
    public Components getComponents() {
        return this.components;
    }

    /**
     * Sets the components that needs to be launched.
     * @param components the set of components.
     */
    public void setComponents(final Components components) {
        this.components = components;
    }

    /**
     * Gets component with the given name.
     * @param componentName the name of the component.
     * @return the component (if any)
     */
    public EZBComponent getComponent(final String componentName) {
        // ask registry if present.
        if (componentManager != null) {
            return componentManager.getComponentRegistry().getComponent(componentName);
        }
        return null;
    }

    /**
     * @return Returns the ComponentManager used by this instance.
     */
    public ComponentManager getComponentManager() {
        return componentManager;
    }
}
