/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ServerConfig.java 1121 2006-09-27 08:51:06Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.server;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * Defines a configuration class that can be used to start the embedded EJB3 server.
 * @author Florent Benoit
 */
public class ServerConfig {

    /**
     * Path where to lookup beans.
     */
    private File ejb3Directory = null;


    /**
     * Wait at the end of the start.
     */
    private boolean shouldWait = true;

    /**
     * Init JACC at startup ?
     */
    private boolean initJACC = true;


    /**
     * Use of MBeans.
     */
    private boolean useMBeans = true;

    /**
     * use EasyBeans naming mechanism or one of the embedded server.
     */
    private boolean useNaming = true;

    /**
     * {@link EasyBeansConfigurationExtension} classname list.
     */
    private List<String> extensionFactories = new ArrayList<String>();

    /**
     * Is the Directory scanning activated.
     */
    private boolean directoryScanningEnabled = true;


    /**
     * @return the path for loading/looking beans.
     */
    public File getEjb3Directory() {
        return ejb3Directory;
    }

    /**
     * Sets the path forloading/looking beans.
     * @param ejb3Path the path for loading beans.
     */
    public void setEjb3Path(final String ejb3Path) {
        this.ejb3Directory = new File(ejb3Path);
        if (!ejb3Directory.exists()) {
            throw new IllegalArgumentException("Path '" + ejb3Path + "' doesn't exists.");
        }
    }


    /**
     * @return true if the server should wait when starting embedded server.
     */
    public boolean shouldWait() {
        return shouldWait;
    }

    /**
     * Sets if the server will loop at the end of it's startup.
     * @param shouldWait true/false
     */
    public void setShouldWait(final boolean shouldWait) {
        this.shouldWait = shouldWait;
    }


    /**
     * Use or not the MBeans.
     * @return true if this is the case.
     */
    public boolean isUsingMBeans() {
        return useMBeans;
    }

    /**
     * Sets the value for using MBeans.
     * @param useMBeans the boolean value.
     */
    public void setUseMBeans(final boolean useMBeans) {
        this.useMBeans = useMBeans;
    }

    /**
     * Use or not the EasyBeans naming system.
     * @return true if this is the case.
     */
    public boolean isUsingNaming() {
        return useNaming;
    }

    /**
     * Sets the value for using the EasyBeans naming system.
     * @param useNaming the boolean value.
     */
    public void setUseNaming(final boolean useNaming) {
        this.useNaming = useNaming;
    }

    /**
     * Adds an {@link EasyBeansConfigurationExtension} in the Facory list.
     * @param extension the factory FQN.
     */
    public void addExtensionFactory(final String extension) {
        this.extensionFactories.add(extension);
    }

    /**
     * @return Returns the list of {@link EasyBeansConfigurationExtension}.
     */
    public List<String> getExtensionFactories() {
        return this.extensionFactories;
    }

    /**
     * @return the directoryScanningEnabled
     */
    public boolean isDirectoryScanningEnabled() {
        return directoryScanningEnabled;
    }

    /**
     * @param directoryScanningEnabled the directoryScanningEnabled to set
     */
    public void setDirectoryScanningEnabled(final boolean directoryScanningEnabled) {
        this.directoryScanningEnabled = directoryScanningEnabled;
    }

    /**
     * Init or not JACC at startup.
     * @param initJACC initialization of JACC provider.
     */
    public void setInitJACC(final boolean initJACC) {
        this.initJACC = initJACC;
    }

    /**
     * @return true if JACC provider needs to be initialized at startup.
     */
    public boolean initJACC() {
        return initJACC;
    }
}
